import { EMPTY, pipe, ReplaySubject, Subject } from 'rxjs';
import { catchError, distinctUntilChanged, map, startWith, takeUntil, withLatestFrom } from 'rxjs/operators';
;
const INIT_STORE_ACTION = { type: 'INIT_STORE' };
// }}}
const createStore = (reducersRoot, effectsRoot) => {
    const actions$ = new Subject();
    const state$ = new ReplaySubject(1);
    const destroy$ = new Subject();
    // reducers
    actions$.pipe(withLatestFrom(state$.pipe(startWith(void 0))), map(([action, state]) => reducersRoot(action, state)), distinctUntilChanged(), suppressAndLogError('[redogs:store]'), takeUntil(destroy$)).subscribe(state$);
    // effects
    if (typeof effectsRoot == 'function') {
        effectsRoot(actions$, state$).pipe(suppressAndLogError('[redogs:effects]'), takeUntil(destroy$))
            .subscribe(actions$);
    }
    // init store
    actions$
        .next(INIT_STORE_ACTION);
    return {
        state$: state$.asObservable(),
        dispatch(action) {
            actions$.next(action);
        },
        destroy$
    };
};
export { INIT_STORE_ACTION, createStore };
// Helpers
function suppressAndLogError(msgPrefix) {
    return pipe(catchError(err => {
        // NOTE: basically, we're suppressing errors here
        if (console && typeof console.error == 'function') {
            console.error(msgPrefix, err);
        }
        return EMPTY;
    }));
}
