import { __read } from "tslib";
import { EMPTY, pipe, ReplaySubject, Subject } from 'rxjs';
import { catchError, distinctUntilChanged, map, startWith, takeUntil, withLatestFrom } from 'rxjs/operators';
;
var INIT_STORE_ACTION = { type: 'INIT_STORE' };
// }}}
var createStore = function (reducersRoot, effectsRoot) {
    var actions$ = new Subject();
    var state$ = new ReplaySubject(1);
    var destroy$ = new Subject();
    // reducers
    actions$.pipe(withLatestFrom(state$.pipe(startWith(void 0))), map(function (_a) {
        var _b = __read(_a, 2), action = _b[0], state = _b[1];
        return reducersRoot(action, state);
    }), distinctUntilChanged(), suppressAndLogError('[redogs:store]'), takeUntil(destroy$)).subscribe(state$);
    // effects
    if (typeof effectsRoot == 'function') {
        effectsRoot(actions$, state$).pipe(suppressAndLogError('[redogs:effects]'), takeUntil(destroy$))
            .subscribe(actions$);
    }
    // init store
    actions$
        .next(INIT_STORE_ACTION);
    return {
        state$: state$.asObservable(),
        dispatch: function (action) {
            actions$.next(action);
        },
        destroy$: destroy$
    };
};
export { INIT_STORE_ACTION, createStore };
// Helpers
function suppressAndLogError(msgPrefix) {
    return pipe(catchError(function (err) {
        // NOTE: basically, we're suppressing errors here
        if (console && typeof console.error == 'function') {
            console.error(msgPrefix, err);
        }
        return EMPTY;
    }));
}
